# -*- coding: utf-8 -*-
import click
from click import argument, option, types  # noqa
import contextlib
from modelhub.core import conf


@click.group()
def main():
    "Modelhub - a Model management system"
    pass


def register(name):
    def wrapper(command_cls):
        assert issubclass(command_cls, BaseCommand)
        command_cls.name = name

        @contextlib.wraps(command_cls.run)
        def command(*args, **kwargs):
            debug = kwargs.pop("debug")
            verbose = kwargs.pop("verbose")
            command = command_cls(verbose)
            if debug:
                import ipdb  # noqa
                with ipdb.launch_ipdb_on_exception():
                    command.run(*args, **kwargs)
            else:
                command.run(*args, **kwargs)
        for arg in reversed(command_cls.add_arguments()):
            command = arg(command)
        main.add_command(click.command(command_cls.name)(command))
        return command_cls
    return wrapper


class BaseCommand():

    def __init__(self, verbose=False):
        self.verbose = verbose

    @staticmethod
    def echo(*args, **kwargs):
        return click.echo(*args, **kwargs)

    arguments = []

    @classmethod
    def add_arguments(cls):
        return cls.arguments + [
            option("--debug", is_flag=True, help="Debug Mode"),
            option("-v", "--verbose", is_flag=True, help="verbose Mode")
        ]

    def get_user_info(self):
        if conf.USER_NAME is None or conf.USER_EMAIL is None:
            self.echo("""You have to set user infomation on ~/.modelhubrc
Example:
[user]
name = Jack Jones
email = jack@example.com
                """)
            raise ValueError
        return conf.USER_NAME, conf.USER_EMAIL



# encoding=utf8
# author=spenly
# mail=i@spenly.com
from .base import ApiModel
from abc import abstractproperty, abstractmethod


class ParentApiModel(ApiModel):
    TYPE = "TF"
    INPUTS_SAMPLE = None
    OUTPUTS_SAMPLE = None
    model_instances = {}

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        super().__init__(*args, **kwargs)

    @abstractproperty
    def option_key(self):
        raise NotImplementedError

    @property
    def default_key(self):
        return None

    def prepare(self):
        super().prepare()
        self.submodels = self.prepare_submodels(*self.args, **self.kwargs)
        if not isinstance(self.submodels, dict) or not all(isinstance(submodel, ApiModel) for submodel in self.submodels.values()):
            raise InvalidSubModel("prepare_submodels must return a dict with key as input type and value as ApiModel instance")
        if self.default_key and self.default_key not in self.submodels:
            raise InvalidSubModel("default_key %s not exist in prepare_submodels" % self.default_key)

    @abstractmethod
    def prepare_submodels(self):
        """
        return {
            "chemistry": translate_cnen_claim_chem_model(*args, **kwargs),
            "electronic": translate_cnen_claim_elec_model(*args, **kwargs),
            "fite": translate_cnen_claim_fite_model(*args, **kwargs),
            "human": translate_cnen_claim_huma_model(*args, **kwargs),
            "mechanic": translate_cnen_claim_mech_model(*args, **kwargs),
            "performance": translate_cnen_claim_perf_model(*args, **kwargs),
            "physical": translate_cnen_claim_phys_model(*args, **kwargs),
        }
        """
        raise NotImplementedError

    def is_ready(self):
        return all(submodel.is_ready() for submodel in self.submodels)

    def validate_input_data(self, raw_input):
        if self.option_key not in raw_input and self.default_key is None:
            raise self.InvalidValueInput("input data need key '%s'" % self.option_key)
        submodel_type = raw_input.get(self.option_key, self.default_key)
        if submodel_type not in self.submodels:
            raise self.InvalidValueInput("%s: %s not support" % (self.option_key, submodel_type))
        if raw_input[self.option_key] not in self.submodels:
            msg = "invalid %s value, should be one of [ %s ]"
            msg = msg % (self.option_key, ",".join(self.submodels.keys()))
            raise self.InvalidValueInput(msg)
        return True

    def run_model(self, preprocessed_data):
        submodel_type = preprocessed_data.get(self.option_key, self.default_key)
        submodel = self.submodels[submodel_type]
        return submodel.run(preprocessed_data)


class InvalidSubModel(Exception):
    pass


ParentBaseModel = ParentApiModel



# -*- coding: utf-8 -*-
from . import info, submit, create, checkout, list, delete, config, version, copy, path, runserver, export, inspect, putup  # noqa
from .base import main

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import configparser
import sys


def load_conf(path, target=None):
    parser = configparser.ConfigParser()
    parser.read(path)
    res = {
        ("%s_%s" % (section_name, key)).upper(): value
        for section_name, section in parser.items()
        for key, value in section.items()
    }
    if target:
        target.update(res)
    else:
        return res


def write_conf(path, **values):
    parser = configparser.ConfigParser()
    parser.read(path)
    if not values:
        return
    for k, v in values.items():
        section, key = k.split("_", 1)
        if section not in parser:
            parser.add_section(section)
        parser.set(section, key, v)
    with open(path, "w") as f:
        parser.write(f)


class cached_property(object):  # noqa
    """A read only property that caches the initially computed value.

    This descriptor will only call the provided ``fget`` function once.
    Subsequent access to this property will return the cached value.

    """

    def __init__(self, fget):
        self._fget = fget

    def __get__(self, obj, cls):
        if obj is None:
            return self
        else:
            computed_value = self._fget(obj)
            obj.__dict__[self._fget.__name__] = computed_value
            return computed_value

if sys.version_info.major < 3:
    import re
    import tokenize
    import keyword

    def isidentifier(string):
        return not keyword.iskeyword(string) and re.match(tokenize.Name + '$', string)

    def mkdir(path):
        import distutils.dir_util
        distutils.dir_util.mkpath(path)
else:
    isidentifier = str.isidentifier

    def mkdir(path):
        import pathlib
        pathlib.Path(path).mkdir(parents=True, exist_ok=True)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_serving/apis/prediction_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow_serving.apis import classification_pb2 as tensorflow__serving_dot_apis_dot_classification__pb2
from tensorflow_serving.apis import get_model_metadata_pb2 as tensorflow__serving_dot_apis_dot_get__model__metadata__pb2
from tensorflow_serving.apis import inference_pb2 as tensorflow__serving_dot_apis_dot_inference__pb2
from tensorflow_serving.apis import predict_pb2 as tensorflow__serving_dot_apis_dot_predict__pb2
from tensorflow_serving.apis import regression_pb2 as tensorflow__serving_dot_apis_dot_regression__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_serving/apis/prediction_service.proto',
  package='tensorflow.serving',
  syntax='proto3',
  serialized_pb=_b('\n0tensorflow_serving/apis/prediction_service.proto\x12\x12tensorflow.serving\x1a,tensorflow_serving/apis/classification.proto\x1a\x30tensorflow_serving/apis/get_model_metadata.proto\x1a\'tensorflow_serving/apis/inference.proto\x1a%tensorflow_serving/apis/predict.proto\x1a(tensorflow_serving/apis/regression.proto2\xfc\x03\n\x11PredictionService\x12\x61\n\x08\x43lassify\x12).tensorflow.serving.ClassificationRequest\x1a*.tensorflow.serving.ClassificationResponse\x12X\n\x07Regress\x12%.tensorflow.serving.RegressionRequest\x1a&.tensorflow.serving.RegressionResponse\x12R\n\x07Predict\x12\".tensorflow.serving.PredictRequest\x1a#.tensorflow.serving.PredictResponse\x12g\n\x0eMultiInference\x12).tensorflow.serving.MultiInferenceRequest\x1a*.tensorflow.serving.MultiInferenceResponse\x12m\n\x10GetModelMetadata\x12+.tensorflow.serving.GetModelMetadataRequest\x1a,.tensorflow.serving.GetModelMetadataResponseB\x03\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[tensorflow__serving_dot_apis_dot_classification__pb2.DESCRIPTOR,tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.DESCRIPTOR,tensorflow__serving_dot_apis_dot_inference__pb2.DESCRIPTOR,tensorflow__serving_dot_apis_dot_predict__pb2.DESCRIPTOR,tensorflow__serving_dot_apis_dot_regression__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
try:
  # THESE ELEMENTS WILL BE DEPRECATED.
  # Please use the generated *_pb2_grpc.py files instead.
  import grpc
  from grpc.beta import implementations as beta_implementations
  from grpc.beta import interfaces as beta_interfaces
  from grpc.framework.common import cardinality
  from grpc.framework.interfaces.face import utilities as face_utilities


  class PredictionServiceStub(object):
    """open source marker; do not remove
    PredictionService provides access to machine-learned models loaded by
    model_servers.
    """

    def __init__(self, channel):
      """Constructor.

      Args:
        channel: A grpc.Channel.
      """
      self.Classify = channel.unary_unary(
          '/tensorflow.serving.PredictionService/Classify',
          request_serializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.SerializeToString,
          response_deserializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.FromString,
          )
      self.Regress = channel.unary_unary(
          '/tensorflow.serving.PredictionService/Regress',
          request_serializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.SerializeToString,
          response_deserializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.FromString,
          )
      self.Predict = channel.unary_unary(
          '/tensorflow.serving.PredictionService/Predict',
          request_serializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.SerializeToString,
          response_deserializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.FromString,
          )
      self.MultiInference = channel.unary_unary(
          '/tensorflow.serving.PredictionService/MultiInference',
          request_serializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.SerializeToString,
          response_deserializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.FromString,
          )
      self.GetModelMetadata = channel.unary_unary(
          '/tensorflow.serving.PredictionService/GetModelMetadata',
          request_serializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.SerializeToString,
          response_deserializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.FromString,
          )


  class PredictionServiceServicer(object):
    """open source marker; do not remove
    PredictionService provides access to machine-learned models loaded by
    model_servers.
    """

    def Classify(self, request, context):
      """Classify.
      """
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def Regress(self, request, context):
      """Regress.
      """
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def Predict(self, request, context):
      """Predict -- provides access to loaded TensorFlow model.
      """
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def MultiInference(self, request, context):
      """MultiInference API for multi-headed models.
      """
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def GetModelMetadata(self, request, context):
      """GetModelMetadata - provides access to metadata for loaded models.
      """
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')


  def add_PredictionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Classify': grpc.unary_unary_rpc_method_handler(
            servicer.Classify,
            request_deserializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.FromString,
            response_serializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.SerializeToString,
        ),
        'Regress': grpc.unary_unary_rpc_method_handler(
            servicer.Regress,
            request_deserializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.FromString,
            response_serializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.SerializeToString,
        ),
        'Predict': grpc.unary_unary_rpc_method_handler(
            servicer.Predict,
            request_deserializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.FromString,
            response_serializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.SerializeToString,
        ),
        'MultiInference': grpc.unary_unary_rpc_method_handler(
            servicer.MultiInference,
            request_deserializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.FromString,
            response_serializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.SerializeToString,
        ),
        'GetModelMetadata': grpc.unary_unary_rpc_method_handler(
            servicer.GetModelMetadata,
            request_deserializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.FromString,
            response_serializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'tensorflow.serving.PredictionService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


  class BetaPredictionServiceServicer(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    """open source marker; do not remove
    PredictionService provides access to machine-learned models loaded by
    model_servers.
    """
    def Classify(self, request, context):
      """Classify.
      """
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def Regress(self, request, context):
      """Regress.
      """
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def Predict(self, request, context):
      """Predict -- provides access to loaded TensorFlow model.
      """
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def MultiInference(self, request, context):
      """MultiInference API for multi-headed models.
      """
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)
    def GetModelMetadata(self, request, context):
      """GetModelMetadata - provides access to metadata for loaded models.
      """
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)


  class BetaPredictionServiceStub(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0."""
    """open source marker; do not remove
    PredictionService provides access to machine-learned models loaded by
    model_servers.
    """
    def Classify(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      """Classify.
      """
      raise NotImplementedError()
    Classify.future = None
    def Regress(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      """Regress.
      """
      raise NotImplementedError()
    Regress.future = None
    def Predict(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      """Predict -- provides access to loaded TensorFlow model.
      """
      raise NotImplementedError()
    Predict.future = None
    def MultiInference(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      """MultiInference API for multi-headed models.
      """
      raise NotImplementedError()
    MultiInference.future = None
    def GetModelMetadata(self, request, timeout, metadata=None, with_call=False, protocol_options=None):
      """GetModelMetadata - provides access to metadata for loaded models.
      """
      raise NotImplementedError()
    GetModelMetadata.future = None


  def beta_create_PredictionService_server(servicer, pool=None, pool_size=None, default_timeout=None, maximum_timeout=None):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_deserializers = {
      ('tensorflow.serving.PredictionService', 'Classify'): tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.FromString,
      ('tensorflow.serving.PredictionService', 'GetModelMetadata'): tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.FromString,
      ('tensorflow.serving.PredictionService', 'MultiInference'): tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.FromString,
      ('tensorflow.serving.PredictionService', 'Predict'): tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.FromString,
      ('tensorflow.serving.PredictionService', 'Regress'): tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.FromString,
    }
    response_serializers = {
      ('tensorflow.serving.PredictionService', 'Classify'): tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.SerializeToString,
      ('tensorflow.serving.PredictionService', 'GetModelMetadata'): tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.SerializeToString,
      ('tensorflow.serving.PredictionService', 'MultiInference'): tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.SerializeToString,
      ('tensorflow.serving.PredictionService', 'Predict'): tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.SerializeToString,
      ('tensorflow.serving.PredictionService', 'Regress'): tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.SerializeToString,
    }
    method_implementations = {
      ('tensorflow.serving.PredictionService', 'Classify'): face_utilities.unary_unary_inline(servicer.Classify),
      ('tensorflow.serving.PredictionService', 'GetModelMetadata'): face_utilities.unary_unary_inline(servicer.GetModelMetadata),
      ('tensorflow.serving.PredictionService', 'MultiInference'): face_utilities.unary_unary_inline(servicer.MultiInference),
      ('tensorflow.serving.PredictionService', 'Predict'): face_utilities.unary_unary_inline(servicer.Predict),
      ('tensorflow.serving.PredictionService', 'Regress'): face_utilities.unary_unary_inline(servicer.Regress),
    }
    server_options = beta_implementations.server_options(request_deserializers=request_deserializers, response_serializers=response_serializers, thread_pool=pool, thread_pool_size=pool_size, default_timeout=default_timeout, maximum_timeout=maximum_timeout)
    return beta_implementations.server(method_implementations, options=server_options)


  def beta_create_PredictionService_stub(channel, host=None, metadata_transformer=None, pool=None, pool_size=None):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0"""
    request_serializers = {
      ('tensorflow.serving.PredictionService', 'Classify'): tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.SerializeToString,
      ('tensorflow.serving.PredictionService', 'GetModelMetadata'): tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.SerializeToString,
      ('tensorflow.serving.PredictionService', 'MultiInference'): tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.SerializeToString,
      ('tensorflow.serving.PredictionService', 'Predict'): tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.SerializeToString,
      ('tensorflow.serving.PredictionService', 'Regress'): tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.SerializeToString,
    }
    response_deserializers = {
      ('tensorflow.serving.PredictionService', 'Classify'): tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.FromString,
      ('tensorflow.serving.PredictionService', 'GetModelMetadata'): tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.FromString,
      ('tensorflow.serving.PredictionService', 'MultiInference'): tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.FromString,
      ('tensorflow.serving.PredictionService', 'Predict'): tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.FromString,
      ('tensorflow.serving.PredictionService', 'Regress'): tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.FromString,
    }
    cardinalities = {
      'Classify': cardinality.Cardinality.UNARY_UNARY,
      'GetModelMetadata': cardinality.Cardinality.UNARY_UNARY,
      'MultiInference': cardinality.Cardinality.UNARY_UNARY,
      'Predict': cardinality.Cardinality.UNARY_UNARY,
      'Regress': cardinality.Cardinality.UNARY_UNARY,
    }
    stub_options = beta_implementations.stub_options(host=host, metadata_transformer=metadata_transformer, request_serializers=request_serializers, response_deserializers=response_deserializers, thread_pool=pool, thread_pool_size=pool_size)
    return beta_implementations.dynamic_stub(channel, 'tensorflow.serving.PredictionService', cardinalities, options=stub_options)
except ImportError:
  pass
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
"""
Example

class Model(TFBaseModel):
    model_name="language_detect"
"""
import numpy as np
from .base import ApiModel
from functools import partial
from modelhub.core.utils import cached_property
import modelhub.apis
from modelhub.utils.numpy import pad_constant
from modelhub.core.models import Model

MAX_MESSAGE_LENGTH = int(1 * 1024 * 1024 * 1024)

GRPC_OPTIONS = [
    ('grpc.max_send_message_length', MAX_MESSAGE_LENGTH),
    ('grpc.max_receive_message_length', MAX_MESSAGE_LENGTH)
]


class TFApiModel(ApiModel):
    TYPE = "TF"

    local_model_version = None

    @cached_property
    def default_schema_name(self):
        import tensorflow as tf
        return tf.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY

    def __init__(
        self,
        run_mode="local",
        hostport=None,
        model_name=None,
        timeout_seconds=5,
        saved_model_path=None,
        local_debug=False,
        tf_debug=False,
        tf_timeline=False,
        local_model_version=None,
        **kwargs
    ):
        super().__init__(**kwargs)
        if model_name is not None:
            self.model_name = model_name
        if local_model_version is not None:
            self.local_model_version = local_model_version

        assert self.model_name
        assert run_mode in ("local", "remote")

        self._run_mode = run_mode

        self._modelhub_model = Model.get_local(self.model_name)
        self._modelhub_version = self._modelhub_model.get_version(self.local_model_version) if self.local_model_version else self._modelhub_model.latest_version
        self._tf_inputs_schema = self._parse_tf_inputs_schema(self._modelhub_version)

        if run_mode == "remote":
            assert hostport, "Must provide hostport when run_mode is remote"
            assert isinstance(hostport, str) or callable(hostport), "hostport can not be type %s, must be a str or a callable" % type(hostport)
            self._prepare_remote(timeout_seconds, hostport)
        elif run_mode == "local":
            self._prepare_local(saved_model_path, local_debug, tf_debug, tf_timeline)
            if local_debug:
                kwargs["verbose"] = True

    def prepare(self):
        pass

    def _prepare_remote(self, timeout_seconds, hostport):
        self.timeout_seconds = timeout_seconds
        from tensorflow_serving.apis import predict_pb2
        from tensorflow.python.framework import tensor_util
        self.PredictRequest = predict_pb2.PredictRequest
        self.tensor_util = tensor_util

        self._hostport = hostport

        import grpc
        from tensorflow_serving.apis import prediction_service_pb2
        stub_builder = lambda hostport_str: prediction_service_pb2.PredictionServiceStub(grpc.insecure_channel(hostport_str, options=GRPC_OPTIONS))

        if isinstance(hostport, str):
            stub = stub_builder(hostport)
            self._stub_fn = lambda: stub
        elif callable(hostport):
            # keep stub in cache in order to maintain long connection
            from functools import lru_cache
            stub_builder = lru_cache(maxsize=100)(stub_builder)

            def _stub_fn():
                hostport_str = hostport(self.model_name, version=self.local_model_version)
                # if hostport_str in stub_cache:
                #     stub = stub_cache[hostport_str]
                # else:
                #     stub = stub_cache.setdefault(hostport_str, stub_builder(hostport_str))
                stub = stub_builder(hostport_str)
                self.log_debug("get stub %s %s", hostport_str, stub)
                return stub

            self._stub_fn = _stub_fn
        else:
            # should not reach here
            raise ValueError("incorrect type of hostport")
        # self._version = Model.get(self.model_name).latest_version

    @property
    def stub(self):
        return self._stub_fn()

    def _prepare_local(self, saved_model_path, local_debug, tf_debug, tf_timeline):
        saved_model_path = saved_model_path or modelhub.apis.get_model_path(self.model_name, version=self.local_model_version, ensure_newest_version=local_debug)
        # from tensorflow. import saved_model_utils
        import tensorflow as tf
        sess = tf.Session(graph=tf.Graph())
        tag_set = ["serve"]
        try:
            meta_graph = tf.saved_model.loader.load(sess, tag_set, saved_model_path)
        except ValueError as e:
            # See https://github.com/tensorflow/tensorflow/issues/10130#issuecomment-303468724
            if e.args and e.args[0].startswith("No op named"):
                sess.close()
                dir(tf.contrib)
                sess = tf.Session(graph=tf.Graph())
                meta_graph = tf.saved_model.loader.load(sess, tag_set, saved_model_path)
            else:
                raise

        if tf_debug:
            from tensorflow.python.debug.wrappers import local_cli_wrapper

            sess = local_cli_wrapper.LocalCLIDebugWrapperSession(sess)
        self._sess = sess
        self._predict_local = {
            signature_name: self._build_callable(sess, signature_def, tf_timeline)
            for signature_name, signature_def in meta_graph.signature_def.items()
        }
        self._predict_local[None] = self._predict_local[self.default_schema_name]

    def _build_callable(self, sess, signature_def, tf_timeline):
        inputs_names = []
        input_tensor_names = []
        for input_name, input_def in signature_def.inputs.items():
            inputs_names.append(input_name)
            input_tensor_names.append(input_def.name)

        outputs_names = []
        output_tensor_names = []
        for output_name, output_def in signature_def.outputs.items():
            outputs_names.append(output_name)
            output_tensor_names.append(output_def.name)

        if not tf_timeline:
            _callable = sess.make_callable(fetches=output_tensor_names, feed_list=input_tensor_names)
        else:
            import tensorflow as tf
            from uuid import uuid4
            from tensorflow.python.client import timeline

            tf_callable = sess.make_callable(fetches=output_tensor_names, feed_list=input_tensor_names, accept_options=True)
            tf_options = tf.RunOptions(trace_level=tf.RunOptions.FULL_TRACE)

            def _callable(*inputs):
                run_metadata = tf.RunMetadata()
                res = tf_callable(*inputs, run_metadata=run_metadata, options=tf_options)

                with open('/tmp/timeline.%s.json' % uuid4(), 'w') as f:
                    f.write(timeline.Timeline(run_metadata.step_stats).generate_chrome_trace_format(show_memory=True))
                return res

        def callable(inputs):
            inputs = [inputs[input_name] for input_name in inputs_names]

            return dict(zip(outputs_names, _callable(*inputs)))
        return callable

    @staticmethod
    def tensorshape2tuple(tensorshape):
        return [dim.size for dim in tensorshape.dim]

    def _parse_tf_inputs_schema(self, modelhub_version):
        from tensorflow.python.framework.dtypes import _TF_TO_NP

        saved_model_info = modelhub_version.saved_model_info
        return {
            signature_name: {
                input_name: (_TF_TO_NP[input_def.dtype], self.tensorshape2tuple(input_def.tensor_shape))
                for input_name, input_def in signature_def.inputs.items()
            }
            for signature_name, signature_def in saved_model_info.signature_def.items()
        }

    @staticmethod
    def verify_shape(value, shape):
        if len(value.shape) != len(shape):
            raise ValueError("incompatible rank")
        for i, (dim, def_dim) in enumerate(zip(value.shape, shape)):
            if def_dim == -1 or def_dim == dim:
                continue
            raise ValueError("unequal shape on rank %s" % i)

    def _build_request(self, inputs, schema):
        request = self.PredictRequest()
        request.model_spec.name = self.model_name

        if schema:
            request.model_spec.signature_name = schema
        else:
            schema = "serving_default"

        inputs_schema = self._tf_inputs_schema[schema]
        for key, value in inputs.items():
            input_dtype, input_shape = inputs_schema[key]
            value = np.array(value, dtype=input_dtype)
            self.verify_shape(value, input_shape)
            request.inputs[key].CopyFrom(self.tensor_util.make_tensor_proto(value))
        return request

    def _parse_result(self, response):
        return {key: self.tensor_util.MakeNdarray(tensor) for key, tensor in response.outputs.items()}

    def _predict_sync(self, inputs, schema=None):
        self.log_debug("send %s", inputs)
        request = self._build_request(inputs, schema)
        response = self.stub.Predict(request, self.timeout_seconds)
        result = self._parse_result(response)
        self.log_debug("recv %s", result)
        return result

    def _callback(self, future, callback, inputs):
        parsed_outputs = self._get_parsed_outputs_from_future(future, inputs=inputs)
        self.log_debug("recv %s %s", id(future), parsed_outputs)
        callback and callback(inputs, parsed_outputs)

    def _get_parsed_outputs_from_future(self, future, inputs=None):
        try:
            response = future.result()
        except Exception as e:
            self.on_error(inputs, e)
            raise
        return self._parse_result(response)

    def on_error(self, inputs, exception):
        self.log_error("Error happended for inputs: %s \n %s", inputs, exception)

    def _predict_async(self, inputs, callback=None, schema=None):
        request = self._build_request(inputs, schema)
        future = self.stub.Predict.future(request, self.timeout_seconds)
        self.log_debug("send %s %s", id(future), inputs)
        if callback is not None or self.verbose:
            callback = partial(self._callback, callback=callback, inputs=inputs)
            future.add_done_callback(callback)

        return future

    def is_ready(self):
        return True

    def run_model(self, preprocessed_item):
        schema = preprocessed_item.pop("schema", [None])[0]
        if self._run_mode == "remote":
            return self._predict_sync(preprocessed_item, schema=schema)
        else:
            return self._predict_local[schema](preprocessed_item)

    def run_batch(self, input_generator):
        """
        data should be something like
        [{
            "input1":value,
            "input2":value
        },{
            "input1":value2,
            "input2":value2,
        }]

        """
        # data = self._input_maybe_list(data)
        if self._run_mode == "remote":
            return self._run_batch_remote(input_generator)
        else:
            return self._run_batch_local(input_generator)

    def _run_batch_remote(self, input_generator):
        futures = []
        preprocessed_items = []
        origin_items = []

        for item in input_generator:

            self.validate_input_data(item)
            preprocessed_item = self.preprocess(item)
            schema = preprocessed_item.pop("schema", [None])[0]

            origin_items.append(item)
            preprocessed_items.append(preprocessed_item)
            futures.append(self._predict_async(preprocessed_item, schema=schema))

        results = [
            self.postprocess(self._get_parsed_outputs_from_future(future), origin_item, preprocessed_item)
            for future, origin_item, preprocessed_item in zip(futures, origin_items, preprocessed_items)
        ]
        return results

    def _run_batch_local(self, input_generator):
        preprocessed_items = []
        origin_items = []

        for item in input_generator:
            preprocessed_item = self.preprocess(item)
            schema = preprocessed_item.pop("schema", [None])[0]

            origin_items.append(item)
            preprocessed_items.append(preprocessed_item)

        if not preprocessed_items:
            return []
        batch_size = len(preprocessed_items)
        inputs = {
            key: self._merge_inputs_of_key(preprocessed_items, key)
            for key in preprocessed_item
        }
        outputs = self._predict_local[schema](inputs)
        splited_outputs = {
            key: np.array_split(output, batch_size)
            for key, output in outputs.items()
        }
        keys = splited_outputs.keys()
        results = []
        for origin_item, preprocessed_item, *each_outputs_list in zip(origin_items, preprocessed_items, *splited_outputs.values()):
            each_output = dict(zip(keys, each_outputs_list))
            results.append(self.postprocess(each_output, origin_item, preprocessed_item))
        return results

    def _dynamic_padding(self, array_list):
        if len(array_list) < 2:
            return array_list
        sample_item = array_list[0]
        if not isinstance(array_list, np.ndarray):
            array_list = [np.array(item) for item in array_list]
            sample_item = array_list[0]

        max_shape = list(sample_item.shape[1:])

        for item in array_list:
            for dim, size in enumerate(item.shape[1:]):
                if max_shape[dim] < size:
                    max_shape[dim] = size
        return np.concatenate([
            pad_constant(
                item,
                [(0, 0)] + [(0, max_size - size) for max_size, size in zip(max_shape, item.shape[1:])],
                constant_values=self._get_padding_value(item)
            )
            for item in array_list
        ])

    def _get_padding_value(self, array):
        dtype = array.dtype.type
        if issubclass(dtype, (np.character)):
            return array.flat[:1]
        else:
            return dtype(0)

    def _merge_inputs_of_key(self, items, key):
        datas = [item[key] for item in items]
        return self._dynamic_padding(datas)

    def _pack_data(self, data):
        """
        data -> {
            "input1":value,
            "input2":value
        }
        return -> {
            "input1":[value],
            "input2":[value],
        }
        """
        return {
            k: [v]
            for k, v in data.items()
        }

    def _unpack_data(self, outputs):
        """
        outputs -> {
            "result1":[value]
        }
        return -> {
            "result1":value
        }
        """
        return {
            k: v[0]
            for k, v in outputs.items()
        }

    def validate_input_data(self, raw_input):
        pass

    def api_schema(self):
        # TODO
        return {}

    def _schema_to_docstring(self, schema):
        # TODO
        return schema

    def preprocess(self, raw_input):
        """
        raw_input -> user input, suggest format like this{
            "input1:value,
            "input2":value
        }
        return:
        {
            "input1":[value],
            "input2":[value]
        }
        """
        return self._pack_data(raw_input)

    def postprocess(self, tf_output, raw_input, preprocessed_item):
        """
        tf_output -> {
            "output1":[value],
            "output2":[value]
        }
        raw_input -> one item of user input data
        preprocessed_item-> {
            "input1":[value],
            "input2":[value]
        }
        return:
        {
            "output1":value,
            "output2":value
        }

        """
        return self._unpack_data(tf_output)


TFBaseModel = TFApiModel



# -*- coding: utf-8 -*-

import os
import io
import hashlib
import json
import threading
import tempfile
import sys
import shutil

import botocore
import boto3

from modelhub import __version__
from . import conf
from .utils import cached_property, mkdir
from .exceptions import VersionError

# from collections import OrderedDict


# class API(object):

#     def __init__(self):
#         self.base_path = urllib.parse.urljoin(conf.API_URI, "model")

#     def __getattr__(self, attr):
#         url = urllib.parse.urljoin(self.base_path, attr)

#         @contextlib.wraps(session.post)
#         def post(*args, **kwargs):
#             return session.post(url, *args, **kwargs)
#         return post


def get_path_last_child(path):
    return list(filter(None, path.split("/")))[-1]


class ProgressPercentage(object):

    def __init__(self, size):
        self._size = size
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._seen_so_far += bytes_amount
            if self._size:
                percentage = (self._seen_so_far / self._size) * 100
                to_write = "\r>>%s / %s  (%.2f%%)" % (
                    self._seen_so_far, self._size,
                    percentage)
            else:
                to_write = "\r>>%s"
            sys.stdout.write(to_write)
            sys.stdout.flush()
            if self._seen_so_far == self._size:
                sys.stdout.write("\n")


class API(object):

    @cached_property
    def session(self):
        assert all([
            conf.AWS_ACCESS_KEY_ID,
            conf.AWS_SECRET_ACCESS_KEY,
            conf.AWS_REGION_NAME,
        ]), "AWS S3 information is required, check if these information exists in `~/.modelhubrc` or use `modelhub config` to set"
        return boto3.session.Session(
            aws_access_key_id=conf.AWS_ACCESS_KEY_ID,
            aws_secret_access_key=conf.AWS_SECRET_ACCESS_KEY,
            region_name=conf.AWS_REGION_NAME,
        )

    @cached_property
    def s3(self):
        return self.session.resource('s3')

    @cached_property
    def bucket(self):
        return self.s3.Bucket(conf.AWS_BUCKET_NAME)

    @cached_property
    def client(self):
        return self.session.client("s3")

    @cached_property
    def server_info(self):
        return self.get_json("modelhub_server_info.json")

    def _get_aws_full_path(self, *paths):
        return os.path.join(conf.AWS_BASE_PATH, *paths)

    def ensure_version(self, key):
        from distutils.version import LooseVersion
        require_version = LooseVersion(self.server_info.get("min_%s_version" % key))
        if require_version is None:
            return
        if require_version > LooseVersion(__version__):
            raise VersionError("%s need modelhub version higher than %s, now is %s" % (key, require_version, __version__))

    def _ls_request(self, full_path, recursive=False):
        kwargs = {} if recursive else {"Delimiter": "/"}
        return api.client.list_objects_v2(Bucket=conf.AWS_BUCKET_NAME, Prefix=full_path, StartAfter=full_path, **kwargs)

    def ls_subfolder(self, *paths):
        full_path = self._get_aws_full_path(*paths)
        # TODO
        return [os.path.relpath(item["Prefix"], full_path) for item in self._ls_request(full_path).get("CommonPrefixes", [])]

    def ls_file(self, *paths, **kwargs):
        recursive = kwargs.get("recursive", False)
        full_path = self._get_aws_full_path(*paths)
        if not full_path.endswith("/"):
            full_path += "/"
        res = self._ls_request(full_path, recursive=recursive).get("Contents", [])
        for item in res:
            item["name"] = os.path.basename(item["Key"])
            item["path"] = os.path.relpath(item["Key"], full_path)

        return res

    def copy_dir(self, old_name, new_name):
        for file in self.ls_file(old_name, recursive=True):
            old_file_name = os.path.join(old_name, file["path"])
            new_file_name = os.path.join(new_name, file["path"])
            self.copy_file(old_file_name, new_file_name)

    def copy_file(self, old_name, new_name):
        # print("copy", old_name, new_name)
        old_full_name = self._get_aws_full_path(old_name)
        new_full_name = self._get_aws_full_path(new_name)
        api.client.copy_object(
            Bucket=conf.AWS_BUCKET_NAME,
            CopySource={'Bucket': conf.AWS_BUCKET_NAME, 'Key': old_full_name},
            Key=new_full_name
        )
        # api.client.delete_object(Bucket=conf.AWS_BUCKET_NAME, Key=old_full_name)

    def get_file(self, fp, *paths, **kwargs):
        md5 = kwargs.get("md5", None)
        progressbar = kwargs.get("progressbar", False)
        size = kwargs.get("size", None)
        auto_copy = kwargs.get("auto_copy", True)
        full_path = self._get_aws_full_path(*paths)
        if progressbar:
            callback = ProgressPercentage(size)
        else:
            callback = None

        if md5 is not None and auto_copy:
            _fp = fp
            fp = tempfile.TemporaryFile()
        try:
            self.bucket.download_fileobj(full_path, fp, Callback=callback)
        except botocore.exceptions.ClientError as e:
            if e.response.get("Error", {}).get("Code") == "404":
                raise self.FileDoesNotExist(full_path)
            raise
        fp.seek(0)
        if md5 is not None:
            new_md5 = self.md5sum(fp)
            if md5 != new_md5:
                raise self.FileCorruptError(full_path, new_md5)
            if auto_copy:
                self.local_copy_file(fp, _fp)
                fp.close()
                fp = _fp
        return fp

    def get_json(self, *paths):
        with io.BytesIO() as f:
            self.get_file(f, *paths)
            return json.loads(f.read().decode())

    def upload_file(self, fp, *paths, **kwargs):
        progressbar = kwargs.get("progressbar", False)
        full_path = self._get_aws_full_path(*paths)
        if progressbar:
            size = self.get_fp_size(fp)
            callback = ProgressPercentage(size)
        else:
            callback = None
        self.bucket.upload_fileobj(fp, full_path, Callback=callback)

    def upload_json(self, content, *paths, **kwargs):
        with io.BytesIO() as f:
            json_str = json.dumps(content, **kwargs)
            f.write(json_str.encode())
            f.seek(0)
            self.upload_file(f, *paths)

    def delete_file(self, *paths, **kwargs):
        recursive = kwargs.get("recursive", False)
        full_path = self._get_aws_full_path(*paths)
        if not full_path.endswith("/"):
            full_path += "/"
        if recursive:
            paths = [file_item["Key"] for file_item in self.ls_file(*paths, recursive=True)]
        else:
            paths = []
        paths.append(full_path)
        # print(paths)
        res = self.bucket.delete_objects(Delete=dict(
            Objects=[
                dict(
                    Key=path
                )
                for path in paths
            ]
        ))
        assert res.get("ResponseMetadata", {}).get("HTTPStatusCode") == 200
        # print("res", res)
        return [item["Key"] for item in res["Deleted"]]

    class FileDoesNotExist(Exception):
        pass

    class FileCorruptError(Exception):
        pass

    @staticmethod
    def md5sum(fp):
        d = hashlib.md5()
        while True:
            buf = fp.read(4096)  # 128 is smaller than the typical filesystem block
            if not buf:
                fp.seek(0)
                break
            d.update(buf)
        return d.hexdigest()

    @staticmethod
    def local_copy_file(from_fp, to_fp):
        while True:
            buf = from_fp.read(4096)  # 128 is smaller than the typical filesystem block
            if not buf:
                break
            to_fp.write(buf)
        to_fp.seek(0)
        from_fp.seek(0)

    @staticmethod
    def get_fp_size(fp):
        fp.seek(0, os.SEEK_END)
        res = fp.tell()
        fp.seek(0)
        return res

    @staticmethod
    def local_link_dir(source_path, target_path):
        mkdir(target_path)
        for root, dirs, files in os.walk(source_path):
            rel_path = os.path.relpath(root, source_path)
            if rel_path == ".":
                target_root = target_path
            else:
                target_root = os.path.join(target_path, rel_path)
            for dir in dirs:
                os.mkdir(os.path.join(target_root, dir))
            for file in files:
                try:
                    os.link(os.path.join(root, file), os.path.join(target_root, file))
                except OSError as e:
                    if e.errno != 18:
                        raise
                    # print("fallback to copy", file)
                    shutil.copy2(os.path.join(root, file), os.path.join(target_root, file))

api = API()



# -*- coding: utf-8 -*-
from modelhub import __version__
import sys
import os
from .base import BaseCommand, argument, option, types, register   # noqa


@register('version')
class Command(BaseCommand):

    def run(self):
        """Show modelhub version"""
        self.echo("Python:\n\t" + sys.version)
        self.echo("\nModelHub:\n\t%s" % __version__)
        self.echo("\nInstall at:\n\t%s" % os.path.dirname((os.path.dirname(__file__))))



# -*- coding: utf-8 -*-
from .base import BaseCommand, argument, option, register   # noqa
from modelhub.core.models import Model


@register("list")
class Command(BaseCommand):

    arguments = [
        option("--local_only", "-l", is_flag=True)
    ]

    def run(self, local_only):
        """List all models on modelhub."""
        models = Model.all()
        for model in models:
            if local_only and not model.local_exists:
                continue
            self.echo(model.name)



# -*- coding: utf-8 -*-
import numpy as np
import numpy.lib.arraypad as padlib


def pad_constant(array, pad_width, constant_values):
    if not issubclass(array.dtype.type, np.character):
        return np.pad(array, pad_width, "constant", constant_values=constant_values)
    narray = np.array(array)
    pad_width = padlib._validate_lengths(narray, pad_width)
    constant_values = padlib._normalize_shape(narray, constant_values, cast_to_int=False)
    newmat = narray.copy()
    for axis, ((pad_before, pad_after), (before_val, after_val)) \
            in enumerate(zip(pad_width, constant_values)):
        newmat = _prepend_const(newmat, pad_before, before_val, axis)
        newmat = _append_const(newmat, pad_after, after_val, axis)
    return newmat


def _prepend_const(arr, pad_amt, val, axis=-1):
    """
    Prepend constant `val` along `axis` of `arr`.

    Parameters
    ----------
    arr : ndarray
        Input array of arbitrary shape.
    pad_amt : int
        Amount of padding to prepend.
    val : scalar
        Constant value to use. For best results should be of type `arr.dtype`;
        if not `arr.dtype` will be cast to `arr.dtype`.
    axis : int
        Axis along which to pad `arr`.

    Returns
    -------
    padarr : ndarray
        Output array, with `pad_amt` constant `val` prepended along `axis`.

    """
    if pad_amt == 0:
        return arr
    padshape = tuple(x if i != axis else pad_amt
                     for (i, x) in enumerate(arr.shape))
    if val == 0:
        return np.concatenate((np.zeros(padshape, dtype=arr.dtype), arr),
                              axis=axis)
    else:
        mat = np.empty(padshape, type(val))
        mat.fill(val)
        return np.concatenate((mat, arr), axis=axis)


def _append_const(arr, pad_amt, val, axis=-1):
    """
    Append constant `val` along `axis` of `arr`.

    Parameters
    ----------
    arr : ndarray
        Input array of arbitrary shape.
    pad_amt : int
        Amount of padding to append.
    val : scalar
        Constant value to use. For best results should be of type `arr.dtype`;
        if not `arr.dtype` will be cast to `arr.dtype`.
    axis : int
        Axis along which to pad `arr`.

    Returns
    -------
    padarr : ndarray
        Output array, with `pad_amt` constant `val` appended along `axis`.

    """
    if pad_amt == 0:
        return arr
    padshape = tuple(x if i != axis else pad_amt
                     for (i, x) in enumerate(arr.shape))
    if val == 0:
        return np.concatenate((arr, np.zeros(padshape, dtype=arr.dtype)),
                              axis=axis)
    else:
        mat = np.empty(padshape, type(val))
        mat.fill(val)
        return np.concatenate((arr, mat), axis=axis)



# -*- coding: utf-8 -*-
from .base import BaseCommand, argument, option, types, register  # noqa
from modelhub.core.models import Model


@register("path")
class Command(BaseCommand):

    arguments = [
        argument("model_name")
    ]

    def run(self, model_name):
        """Show local path of the given model and version"""
        model_name, version = Model.split_name_version(model_name)
        model = Model.get_local(model_name, fallback_to_remote=False)
        if not version:
            version = model.latest_local_version
        else:
            version = model.get_version(version)
            if not version.local_exists:
                raise version.DoesNotExist("Version {version.manifest.seq} dost not exist at local".format(version=version))
        self.echo(version.local_path)



# -*- coding: utf-8 -*-
from modelhub.core import models


def get_model_path(model_name, version=None, ensure_newest_version=False):
    if version:
        model = models.Model.get_local(model_name)
        version = model.get_version(version)
    else:
        if ensure_newest_version:
            model = models.Model.get(model_name)
            version = model.latest_version
        else:
            model = models.Model.get_local(model_name)
            for version in reversed(model.versions):
                if version.local_exists:
                    break
            else:
                version = model.latest_version

    if not version.local_exists:
        version.checkout()
    return version.local_path



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_serving/config/model_server_config.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from tensorflow_serving.config import logging_config_pb2 as tensorflow__serving_dot_config_dot_logging__config__pb2
from tensorflow_serving.sources.storage_path import file_system_storage_path_source_pb2 as tensorflow__serving_dot_sources_dot_storage__path_dot_file__system__storage__path__source__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_serving/config/model_server_config.proto',
  package='tensorflow.serving',
  syntax='proto3',
  serialized_pb=_b('\n3tensorflow_serving/config/model_server_config.proto\x12\x12tensorflow.serving\x1a\x19google/protobuf/any.proto\x1a.tensorflow_serving/config/logging_config.proto\x1aMtensorflow_serving/sources/storage_path/file_system_storage_path_source.proto\"\xa9\x02\n\x0bModelConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tbase_path\x18\x02 \x01(\t\x12\x35\n\nmodel_type\x18\x03 \x01(\x0e\x32\x1d.tensorflow.serving.ModelTypeB\x02\x18\x01\x12\x16\n\x0emodel_platform\x18\x04 \x01(\t\x12i\n\x14model_version_policy\x18\x07 \x01(\x0b\x32K.tensorflow.serving.FileSystemStoragePathSourceConfig.ServableVersionPolicy\x12\x39\n\x0elogging_config\x18\x06 \x01(\x0b\x32!.tensorflow.serving.LoggingConfigJ\x04\x08\x05\x10\x06\"B\n\x0fModelConfigList\x12/\n\x06\x63onfig\x18\x01 \x03(\x0b\x32\x1f.tensorflow.serving.ModelConfig\"\x94\x01\n\x11ModelServerConfig\x12@\n\x11model_config_list\x18\x01 \x01(\x0b\x32#.tensorflow.serving.ModelConfigListH\x00\x12\x33\n\x13\x63ustom_model_config\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x42\x08\n\x06\x63onfig*N\n\tModelType\x12\x1e\n\x16MODEL_TYPE_UNSPECIFIED\x10\x00\x1a\x02\x08\x01\x12\x12\n\nTENSORFLOW\x10\x01\x1a\x02\x08\x01\x12\r\n\x05OTHER\x10\x02\x1a\x02\x08\x01\x42\x03\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,tensorflow__serving_dot_config_dot_logging__config__pb2.DESCRIPTOR,tensorflow__serving_dot_sources_dot_storage__path_dot_file__system__storage__path__source__pb2.DESCRIPTOR,])

_MODELTYPE = _descriptor.EnumDescriptor(
  name='ModelType',
  full_name='tensorflow.serving.ModelType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MODEL_TYPE_UNSPECIFIED', index=0, number=0,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TENSORFLOW', index=1, number=1,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001')),
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=2, number=2,
      options=_descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001')),
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=748,
  serialized_end=826,
)
_sym_db.RegisterEnumDescriptor(_MODELTYPE)

ModelType = enum_type_wrapper.EnumTypeWrapper(_MODELTYPE)
MODEL_TYPE_UNSPECIFIED = 0
TENSORFLOW = 1
OTHER = 2



_MODELCONFIG = _descriptor.Descriptor(
  name='ModelConfig',
  full_name='tensorflow.serving.ModelConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.serving.ModelConfig.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='base_path', full_name='tensorflow.serving.ModelConfig.base_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model_type', full_name='tensorflow.serving.ModelConfig.model_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model_platform', full_name='tensorflow.serving.ModelConfig.model_platform', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model_version_policy', full_name='tensorflow.serving.ModelConfig.model_version_policy', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='logging_config', full_name='tensorflow.serving.ModelConfig.logging_config', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=230,
  serialized_end=527,
)


_MODELCONFIGLIST = _descriptor.Descriptor(
  name='ModelConfigList',
  full_name='tensorflow.serving.ModelConfigList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='tensorflow.serving.ModelConfigList.config', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=529,
  serialized_end=595,
)


_MODELSERVERCONFIG = _descriptor.Descriptor(
  name='ModelServerConfig',
  full_name='tensorflow.serving.ModelServerConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_config_list', full_name='tensorflow.serving.ModelServerConfig.model_config_list', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='custom_model_config', full_name='tensorflow.serving.ModelServerConfig.custom_model_config', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='config', full_name='tensorflow.serving.ModelServerConfig.config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=598,
  serialized_end=746,
)

_MODELCONFIG.fields_by_name['model_type'].enum_type = _MODELTYPE
_MODELCONFIG.fields_by_name['model_version_policy'].message_type = tensorflow__serving_dot_sources_dot_storage__path_dot_file__system__storage__path__source__pb2._FILESYSTEMSTORAGEPATHSOURCECONFIG_SERVABLEVERSIONPOLICY
_MODELCONFIG.fields_by_name['logging_config'].message_type = tensorflow__serving_dot_config_dot_logging__config__pb2._LOGGINGCONFIG
_MODELCONFIGLIST.fields_by_name['config'].message_type = _MODELCONFIG
_MODELSERVERCONFIG.fields_by_name['model_config_list'].message_type = _MODELCONFIGLIST
_MODELSERVERCONFIG.fields_by_name['custom_model_config'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_MODELSERVERCONFIG.oneofs_by_name['config'].fields.append(
  _MODELSERVERCONFIG.fields_by_name['model_config_list'])
_MODELSERVERCONFIG.fields_by_name['model_config_list'].containing_oneof = _MODELSERVERCONFIG.oneofs_by_name['config']
_MODELSERVERCONFIG.oneofs_by_name['config'].fields.append(
  _MODELSERVERCONFIG.fields_by_name['custom_model_config'])
_MODELSERVERCONFIG.fields_by_name['custom_model_config'].containing_oneof = _MODELSERVERCONFIG.oneofs_by_name['config']
DESCRIPTOR.message_types_by_name['ModelConfig'] = _MODELCONFIG
DESCRIPTOR.message_types_by_name['ModelConfigList'] = _MODELCONFIGLIST
DESCRIPTOR.message_types_by_name['ModelServerConfig'] = _MODELSERVERCONFIG
DESCRIPTOR.enum_types_by_name['ModelType'] = _MODELTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ModelConfig = _reflection.GeneratedProtocolMessageType('ModelConfig', (_message.Message,), dict(
  DESCRIPTOR = _MODELCONFIG,
  __module__ = 'tensorflow_serving.config.model_server_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.ModelConfig)
  ))
_sym_db.RegisterMessage(ModelConfig)

ModelConfigList = _reflection.GeneratedProtocolMessageType('ModelConfigList', (_message.Message,), dict(
  DESCRIPTOR = _MODELCONFIGLIST,
  __module__ = 'tensorflow_serving.config.model_server_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.ModelConfigList)
  ))
_sym_db.RegisterMessage(ModelConfigList)

ModelServerConfig = _reflection.GeneratedProtocolMessageType('ModelServerConfig', (_message.Message,), dict(
  DESCRIPTOR = _MODELSERVERCONFIG,
  __module__ = 'tensorflow_serving.config.model_server_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.ModelServerConfig)
  ))
_sym_db.RegisterMessage(ModelServerConfig)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
_MODELTYPE.values_by_name["MODEL_TYPE_UNSPECIFIED"].has_options = True
_MODELTYPE.values_by_name["MODEL_TYPE_UNSPECIFIED"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001'))
_MODELTYPE.values_by_name["TENSORFLOW"].has_options = True
_MODELTYPE.values_by_name["TENSORFLOW"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001'))
_MODELTYPE.values_by_name["OTHER"].has_options = True
_MODELTYPE.values_by_name["OTHER"]._options = _descriptor._ParseOptions(descriptor_pb2.EnumValueOptions(), _b('\010\001'))
_MODELCONFIG.fields_by_name['model_type'].has_options = True
_MODELCONFIG.fields_by_name['model_type']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*

from modelhub.core.models import Model, ModelVersion
from .base import BaseCommand, argument, option, types, register


@register("submit")
class Command(BaseCommand):

    arguments = [
        argument("model_name", type=types.STRING),
        argument("model_path", type=types.Path(file_okay=False, exists=True), default="."),
        option("--comment", "-m", prompt=True, help='model name shoule be a str')
    ]

    def run(self, model_name, model_path, comment):
        """Submit your model to modelhub."""
        try:
            model = Model.get(model_name)
        except Model.DoesNotExist:
            self.echo("Error: model {model_name} not exist, you need execute 'modelhub create {model_name}' ahead".format(model_name=model_name))
            return
        submit_username, submit_email = self.get_user_info()
        try:
            model_version = ModelVersion.create(model, model_path, comment=comment, submit_username=submit_username, submit_email=submit_email)
        except ModelVersion.ValidationError as e:
            self.echo(e.args[0])
        else:
            self.echo("Done submit model {model.name}, version {model_version.manifest.seq}, size {model_version.manifest.compressed_size}".format(**locals()))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: core/protos/model.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


try:
  from tensorflow.core.protobuf import meta_graph_pb2 as tensorflow_dot_core_dot_protobuf_dot_meta__graph__pb2
except ImportError:
  from tensorflow_protos.core.protobuf import meta_graph_pb2 as tensorflow_dot_core_dot_protobuf_dot_meta__graph__pb2

from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='core/protos/model.proto',
  package='modelhub',
  syntax='proto3',
  serialized_pb=_b('\n\x17\x63ore/protos/model.proto\x12\x08modelhub\x1a)tensorflow/core/protobuf/meta_graph.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x9e\x02\n\x0eSavedModelInfo\x12\x0c\n\x04tags\x18\x01 \x03(\t\x12\x1a\n\x12tensorflow_version\x18\x02 \x01(\t\x12\x1e\n\x16tensorflow_git_version\x18\x03 \x01(\t\x12\x41\n\rsignature_def\x18\x04 \x03(\x0b\x32*.modelhub.SavedModelInfo.SignatureDefEntry\x12\x30\n\x0e\x61sset_file_def\x18\x05 \x03(\x0b\x32\x18.tensorflow.AssetFileDef\x1aM\n\x11SignatureDefEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.tensorflow.SignatureDef:\x02\x38\x01\"<\n\x07\x46ileDef\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x11\n\tfile_size\x18\x02 \x01(\x03\x12\x0b\n\x03md5\x18\x03 \x01(\t\"\x86\x03\n\x0cModelVersion\x12\x0b\n\x03seq\x18\x01 \x01(\x05\x12\x0f\n\x07\x63omment\x18\x02 \x01(\t\x12\x17\n\x0fsubmit_username\x18\x03 \x01(\t\x12\x14\n\x0csubmit_email\x18\x04 \x01(\t\x12\x33\n\x0fsubmit_datetime\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0brequire_gpu\x18\x06 \x01(\x08\x12\x13\n\x0b\x61pi_py_name\x18\x07 \x01(\t\x12\x11\n\tfile_name\x18\x08 \x01(\t\x12\x17\n\x0f\x63ompressed_size\x18\t \x01(\x03\x12\x16\n\x0e\x63ompressed_md5\x18\n \x01(\t\x12\x16\n\x0eis_saved_model\x18\x0b \x01(\x08\x12\x32\n\x10saved_model_info\x18\x0c \x03(\x0b\x32\x18.modelhub.SavedModelInfo\x12\x18\n\x10modelhub_version\x18\r \x01(\t\x12 \n\x05\x66iles\x18\x0e \x03(\x0b\x32\x11.modelhub.FileDef\"\xec\x01\n\x05Model\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x12\n\nowner_name\x18\x03 \x01(\t\x12\x13\n\x0bowner_email\x18\x04 \x01(\t\x12(\n\x08versions\x18\x05 \x03(\x0b\x32\x16.modelhub.ModelVersion\x12\x33\n\x0f\x63reate_datetime\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x14last_update_datetime\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestampb\x06proto3')
  ,
  dependencies=[tensorflow_dot_core_dot_protobuf_dot_meta__graph__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_SAVEDMODELINFO_SIGNATUREDEFENTRY = _descriptor.Descriptor(
  name='SignatureDefEntry',
  full_name='modelhub.SavedModelInfo.SignatureDefEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='modelhub.SavedModelInfo.SignatureDefEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='modelhub.SavedModelInfo.SignatureDefEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=323,
  serialized_end=400,
)

_SAVEDMODELINFO = _descriptor.Descriptor(
  name='SavedModelInfo',
  full_name='modelhub.SavedModelInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tags', full_name='modelhub.SavedModelInfo.tags', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tensorflow_version', full_name='modelhub.SavedModelInfo.tensorflow_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tensorflow_git_version', full_name='modelhub.SavedModelInfo.tensorflow_git_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature_def', full_name='modelhub.SavedModelInfo.signature_def', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asset_file_def', full_name='modelhub.SavedModelInfo.asset_file_def', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SAVEDMODELINFO_SIGNATUREDEFENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=400,
)


_FILEDEF = _descriptor.Descriptor(
  name='FileDef',
  full_name='modelhub.FileDef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='modelhub.FileDef.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_size', full_name='modelhub.FileDef.file_size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='md5', full_name='modelhub.FileDef.md5', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=462,
)


_MODELVERSION = _descriptor.Descriptor(
  name='ModelVersion',
  full_name='modelhub.ModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seq', full_name='modelhub.ModelVersion.seq', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='modelhub.ModelVersion.comment', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='submit_username', full_name='modelhub.ModelVersion.submit_username', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='submit_email', full_name='modelhub.ModelVersion.submit_email', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='submit_datetime', full_name='modelhub.ModelVersion.submit_datetime', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='require_gpu', full_name='modelhub.ModelVersion.require_gpu', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='api_py_name', full_name='modelhub.ModelVersion.api_py_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file_name', full_name='modelhub.ModelVersion.file_name', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compressed_size', full_name='modelhub.ModelVersion.compressed_size', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compressed_md5', full_name='modelhub.ModelVersion.compressed_md5', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_saved_model', full_name='modelhub.ModelVersion.is_saved_model', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='saved_model_info', full_name='modelhub.ModelVersion.saved_model_info', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='modelhub_version', full_name='modelhub.ModelVersion.modelhub_version', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='files', full_name='modelhub.ModelVersion.files', index=13,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=855,
)


_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='modelhub.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='modelhub.Model.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='modelhub.Model.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner_name', full_name='modelhub.Model.owner_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner_email', full_name='modelhub.Model.owner_email', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='versions', full_name='modelhub.Model.versions', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='create_datetime', full_name='modelhub.Model.create_datetime', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_update_datetime', full_name='modelhub.Model.last_update_datetime', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=858,
  serialized_end=1094,
)

_SAVEDMODELINFO_SIGNATUREDEFENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_protobuf_dot_meta__graph__pb2._SIGNATUREDEF
_SAVEDMODELINFO_SIGNATUREDEFENTRY.containing_type = _SAVEDMODELINFO
_SAVEDMODELINFO.fields_by_name['signature_def'].message_type = _SAVEDMODELINFO_SIGNATUREDEFENTRY
_SAVEDMODELINFO.fields_by_name['asset_file_def'].message_type = tensorflow_dot_core_dot_protobuf_dot_meta__graph__pb2._ASSETFILEDEF
_MODELVERSION.fields_by_name['submit_datetime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_MODELVERSION.fields_by_name['saved_model_info'].message_type = _SAVEDMODELINFO
_MODELVERSION.fields_by_name['files'].message_type = _FILEDEF
_MODEL.fields_by_name['versions'].message_type = _MODELVERSION
_MODEL.fields_by_name['create_datetime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_MODEL.fields_by_name['last_update_datetime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['SavedModelInfo'] = _SAVEDMODELINFO
DESCRIPTOR.message_types_by_name['FileDef'] = _FILEDEF
DESCRIPTOR.message_types_by_name['ModelVersion'] = _MODELVERSION
DESCRIPTOR.message_types_by_name['Model'] = _MODEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SavedModelInfo = _reflection.GeneratedProtocolMessageType('SavedModelInfo', (_message.Message,), dict(

  SignatureDefEntry = _reflection.GeneratedProtocolMessageType('SignatureDefEntry', (_message.Message,), dict(
    DESCRIPTOR = _SAVEDMODELINFO_SIGNATUREDEFENTRY,
    __module__ = 'core.protos.model_pb2'
    # @@protoc_insertion_point(class_scope:modelhub.SavedModelInfo.SignatureDefEntry)
    ))
  ,
  DESCRIPTOR = _SAVEDMODELINFO,
  __module__ = 'core.protos.model_pb2'
  # @@protoc_insertion_point(class_scope:modelhub.SavedModelInfo)
  ))
_sym_db.RegisterMessage(SavedModelInfo)
_sym_db.RegisterMessage(SavedModelInfo.SignatureDefEntry)

FileDef = _reflection.GeneratedProtocolMessageType('FileDef', (_message.Message,), dict(
  DESCRIPTOR = _FILEDEF,
  __module__ = 'core.protos.model_pb2'
  # @@protoc_insertion_point(class_scope:modelhub.FileDef)
  ))
_sym_db.RegisterMessage(FileDef)

ModelVersion = _reflection.GeneratedProtocolMessageType('ModelVersion', (_message.Message,), dict(
  DESCRIPTOR = _MODELVERSION,
  __module__ = 'core.protos.model_pb2'
  # @@protoc_insertion_point(class_scope:modelhub.ModelVersion)
  ))
_sym_db.RegisterMessage(ModelVersion)

Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), dict(
  DESCRIPTOR = _MODEL,
  __module__ = 'core.protos.model_pb2'
  # @@protoc_insertion_point(class_scope:modelhub.Model)
  ))
_sym_db.RegisterMessage(Model)


_SAVEDMODELINFO_SIGNATUREDEFENTRY.has_options = True
_SAVEDMODELINFO_SIGNATUREDEFENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)




AWS_ACCESS_KEY_ID = ""

AWS_SECRET_ACCESS_KEY = ""

AWS_REGION_NAME = ""

AWS_BUCKET_NAME = ""

AWS_BASE_PATH = "services/modelhub/"

LOG_CONFIG = {
    'version': 1,
    'formatters': {
        'verbose': {'format': "[%(asctime)-15s] [%(levelname)-5s] [%(name)s] %(message)s"},
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'api_model': {
            'handlers': ['console'],
            'level': 'INFO',
        }
    },
}


# encoding=utf8
# author=spenly
# mail=i@spenly.com

from redis import StrictRedis
import re
import socket
import random
from urllib.parse import urlparse


def parse_redis_url(url):
    """
    Parse a redis URL.
    :param url: redis://host:port[,host2:port2,...][/service_name[/db]][?socket_timeout=0.1]
    :return: redis kwargs
    """
    url = urlparse(url)
    if url.scheme != 'redis':
        raise ValueError('Unsupported scheme: {}'.format(url.scheme))

    def parse_host(s):
        if ':' in s:
            host, port = s.split(':', 1)
            port = int(port)
        else:
            host = s
            port = 6379
        return host, port

    if '@' in url.netloc:
        auth, hostspec = url.netloc.split('@', 1)
    else:
        auth = None
        hostspec = url.netloc
    if auth and ':' in auth:
        _, password = auth.split(':', 1)
    else:
        password = None
    host, port = parse_host(hostspec)
    db = int("0" + re.sub("\D", "", url.path))
    kwargs = {"host": host, "port": port, "password": password, "db": db}
    return kwargs


def get_local_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


class AutoDiscovery(object):

    def __init__(self, redis_prefix="", **kwargs):
        self._redis = StrictRedis(**kwargs)
        # e.g. 192_168_42_70
        self._register_ip = get_local_ip().replace(".", "_")
        self._redis_prefix = redis_prefix

    def _build_model_name_ver(self, model_name, version=None):
        if version:
            return "%s_%s@%d" % (self._redis_prefix, model_name, version)
        else:
            return "%s_%s" % (self._redis_prefix, model_name)

    def register(self, model_name, version, hostport, timeout=10):
        """
        register a TF serving info
        :param model_name: model name
        :param version: model version
        :param hostport: GPU instance hostport value, e.g. "192.168.42.70:8500"
        :param timeout: expire time
        :return:
        """
        version = int(version)
        model_name_ver = self._build_model_name_ver(model_name, version)
        key = "%s:%s" % (model_name_ver, self._register_ip)
        self._redis.set(key, value=hostport, ex=timeout)

    def get_host_port(self, model_name, version=None):
        """
        get a TF serving host info
        :param model_name:
        :param version: int or str
        :return:
        """
        if not version:
            model_name_prefix = self._build_model_name_ver(model_name)
            start_index = len(model_name_prefix) + 1
            latest_ver = -1
            # find the latest version
            for item in self._redis.keys("%s*" % model_name_prefix):
                # model_name@ver:ip, e.g.: language_detection@1:10.112.3.54
                item = isinstance(item, bytes) and item.decode("utf8") or item
                end_index = item.index(":")
                cur_ver = int(item[start_index:end_index])
                latest_ver = (cur_ver > latest_ver) and cur_ver or latest_ver
            # version validation
            if latest_ver < 0:
                raise Exception("No available serving versions for model %s" % model_name)
            else:
                version = latest_ver
        version = int(version)
        model_name_ver = self._build_model_name_ver(model_name, version)
        keys = self._redis.keys("%s*" % model_name_ver)
        # value validation, empty value will raise a exception
        if not keys:
            raise Exception("Serving host for %s@%s not exists" % (model_name, version))
        key = random.choice(keys)
        val = self._redis.get(key)
        if isinstance(val, bytes):
            val = val.decode("utf8")
        return val


if __name__ == "__main__":
    # get redis url from param or ENV
    args = parse_redis_url("redis://192.168.44.130")
    ad = AutoDiscovery(**args)
    print("\nregister: test@1=hostport_for_test")
    ad.register("test", "1", "hostport_for_test")
    print("\nget hostport for model test@latest")
    print("value => " + ad.get_host_port("test", None))

    print("\nget host port for model test@1")
    print("value => " + ad.get_host_port("test", version=1))

    print("\nregister: abc@2=hostport_for_abc")
    ad.register("abc", 2, "hostport_for_abc@2")

    print("\nget hostport for model abc@latest")
    print("value => " + ad.get_host_port("abc"))

    print("\nget host port for model test@2 will occur a exception:")
    print("value => " + ad.get_host_port("test", version=2))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_serving/apis/predict.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import tensor_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__pb2
from tensorflow_serving.apis import model_pb2 as tensorflow__serving_dot_apis_dot_model__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_serving/apis/predict.proto',
  package='tensorflow.serving',
  syntax='proto3',
  serialized_pb=_b('\n%tensorflow_serving/apis/predict.proto\x12\x12tensorflow.serving\x1a&tensorflow/core/framework/tensor.proto\x1a#tensorflow_serving/apis/model.proto\"\xe2\x01\n\x0ePredictRequest\x12\x31\n\nmodel_spec\x18\x01 \x01(\x0b\x32\x1d.tensorflow.serving.ModelSpec\x12>\n\x06inputs\x18\x02 \x03(\x0b\x32..tensorflow.serving.PredictRequest.InputsEntry\x12\x15\n\routput_filter\x18\x03 \x03(\t\x1a\x46\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.tensorflow.TensorProto:\x02\x38\x01\"\x9d\x01\n\x0fPredictResponse\x12\x41\n\x07outputs\x18\x01 \x03(\x0b\x32\x30.tensorflow.serving.PredictResponse.OutputsEntry\x1aG\n\x0cOutputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.tensorflow.TensorProto:\x02\x38\x01\x42\x03\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[tensorflow_dot_core_dot_framework_dot_tensor__pb2.DESCRIPTOR,tensorflow__serving_dot_apis_dot_model__pb2.DESCRIPTOR,])




_PREDICTREQUEST_INPUTSENTRY = _descriptor.Descriptor(
  name='InputsEntry',
  full_name='tensorflow.serving.PredictRequest.InputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tensorflow.serving.PredictRequest.InputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorflow.serving.PredictRequest.InputsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=295,
  serialized_end=365,
)

_PREDICTREQUEST = _descriptor.Descriptor(
  name='PredictRequest',
  full_name='tensorflow.serving.PredictRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_spec', full_name='tensorflow.serving.PredictRequest.model_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='tensorflow.serving.PredictRequest.inputs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='output_filter', full_name='tensorflow.serving.PredictRequest.output_filter', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_PREDICTREQUEST_INPUTSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=139,
  serialized_end=365,
)


_PREDICTRESPONSE_OUTPUTSENTRY = _descriptor.Descriptor(
  name='OutputsEntry',
  full_name='tensorflow.serving.PredictResponse.OutputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tensorflow.serving.PredictResponse.OutputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorflow.serving.PredictResponse.OutputsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=454,
  serialized_end=525,
)

_PREDICTRESPONSE = _descriptor.Descriptor(
  name='PredictResponse',
  full_name='tensorflow.serving.PredictResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='outputs', full_name='tensorflow.serving.PredictResponse.outputs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_PREDICTRESPONSE_OUTPUTSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=368,
  serialized_end=525,
)

_PREDICTREQUEST_INPUTSENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_framework_dot_tensor__pb2._TENSORPROTO
_PREDICTREQUEST_INPUTSENTRY.containing_type = _PREDICTREQUEST
_PREDICTREQUEST.fields_by_name['model_spec'].message_type = tensorflow__serving_dot_apis_dot_model__pb2._MODELSPEC
_PREDICTREQUEST.fields_by_name['inputs'].message_type = _PREDICTREQUEST_INPUTSENTRY
_PREDICTRESPONSE_OUTPUTSENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_framework_dot_tensor__pb2._TENSORPROTO
_PREDICTRESPONSE_OUTPUTSENTRY.containing_type = _PREDICTRESPONSE
_PREDICTRESPONSE.fields_by_name['outputs'].message_type = _PREDICTRESPONSE_OUTPUTSENTRY
DESCRIPTOR.message_types_by_name['PredictRequest'] = _PREDICTREQUEST
DESCRIPTOR.message_types_by_name['PredictResponse'] = _PREDICTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PredictRequest = _reflection.GeneratedProtocolMessageType('PredictRequest', (_message.Message,), dict(

  InputsEntry = _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), dict(
    DESCRIPTOR = _PREDICTREQUEST_INPUTSENTRY,
    __module__ = 'tensorflow_serving.apis.predict_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.serving.PredictRequest.InputsEntry)
    ))
  ,
  DESCRIPTOR = _PREDICTREQUEST,
  __module__ = 'tensorflow_serving.apis.predict_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.PredictRequest)
  ))
_sym_db.RegisterMessage(PredictRequest)
_sym_db.RegisterMessage(PredictRequest.InputsEntry)

PredictResponse = _reflection.GeneratedProtocolMessageType('PredictResponse', (_message.Message,), dict(

  OutputsEntry = _reflection.GeneratedProtocolMessageType('OutputsEntry', (_message.Message,), dict(
    DESCRIPTOR = _PREDICTRESPONSE_OUTPUTSENTRY,
    __module__ = 'tensorflow_serving.apis.predict_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.serving.PredictResponse.OutputsEntry)
    ))
  ,
  DESCRIPTOR = _PREDICTRESPONSE,
  __module__ = 'tensorflow_serving.apis.predict_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.PredictResponse)
  ))
_sym_db.RegisterMessage(PredictResponse)
_sym_db.RegisterMessage(PredictResponse.OutputsEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
_PREDICTREQUEST_INPUTSENTRY.has_options = True
_PREDICTREQUEST_INPUTSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_PREDICTRESPONSE_OUTPUTSENTRY.has_options = True
_PREDICTRESPONSE_OUTPUTSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
try:
  # THESE ELEMENTS WILL BE DEPRECATED.
  # Please use the generated *_pb2_grpc.py files instead.
  import grpc
  from grpc.beta import implementations as beta_implementations
  from grpc.beta import interfaces as beta_interfaces
  from grpc.framework.common import cardinality
  from grpc.framework.interfaces.face import utilities as face_utilities
except ImportError:
  pass
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from __future__ import print_function

import six
from .base import BaseCommand, option, argument, register   # noqa
from modelhub.core import conf, utils


@register("config")
class Command(BaseCommand):

    arguments = []

    params = {
        "user_name": "Your Name",
        "user_email": "Your Email",
        "local_data_dir": "Path of data dir",
        "aws_access_key_id": "AWS ACCESS_KEY_ID",
        "aws_secret_access_key": "AWS SECRET_ACCESS_KEY",
        "aws_region_name": "AWS REGION_NAME",
        "aws_bucket_name": "AWS BUCKET_NAME",
        "aws_base_path": "AWS BASE_PATH",
    }

    interactive_keys = [
        "user_name",
        "user_email",
        "aws_access_key_id",
        "aws_secret_access_key",
        "aws_region_name",
        "aws_bucket_name",
    ]

    @classmethod
    def add_arguments(cls):
        return BaseCommand.add_arguments() + [
            option("-i", "--interactive", is_flag=True, help="Enter interactive mode")
        ] + [
            option(
                "--" + key.replace(".", "_"),
                help=help_text
            )
            for key, help_text in cls.params.items()
        ]

    def run(self, interactive=True, **kwargs):
        "Get and set modelhub global options"
        values = {k: v for k, v in kwargs.items() if v}
        input_keys = set(values)
        if interactive:
            for k in self.interactive_keys:
                if k in input_keys:
                    continue
                current_value = values.get(k) or getattr(conf, k.upper(), None)
                new_value = self.get_input(k, current_value)
                values[k] = new_value

        if values:
            values = {k: v for k, v in values.items() if v != getattr(conf, k.upper(), None)}
            if values:
                self.write_config(values)
        else:
            self.show_config()

    def write_config(self, v):
        utils.write_conf(conf.RC_PATH, **v)
        print("saved to {path}".format(path=conf.RC_PATH))
        # for key, value in v.items():
        #     print("{key}:\t{value}".format(key=key, value=value))

    def get_input(self, k, v):
        help_text = "%s:" % self.params[k]
        if v:
            help_text += "[%s]" % v
        help_text += ">>"
        new_v = six.moves.input(help_text).strip()

        return new_v or v

    def show_config(self):
        for k, help_text in self.params.items():
            print("{help_text}({key}):\n    {value}\n".format(
                key=k,
                help_text=help_text,
                value=getattr(conf, k.upper()))
            )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_serving/config/log_collector_config.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_serving/config/log_collector_config.proto',
  package='tensorflow.serving',
  syntax='proto3',
  serialized_pb=_b('\n4tensorflow_serving/config/log_collector_config.proto\x12\x12tensorflow.serving\";\n\x12LogCollectorConfig\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x17\n\x0f\x66ilename_prefix\x18\x02 \x01(\tB\x03\xf8\x01\x01\x62\x06proto3')
)




_LOGCOLLECTORCONFIG = _descriptor.Descriptor(
  name='LogCollectorConfig',
  full_name='tensorflow.serving.LogCollectorConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='tensorflow.serving.LogCollectorConfig.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filename_prefix', full_name='tensorflow.serving.LogCollectorConfig.filename_prefix', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=76,
  serialized_end=135,
)

DESCRIPTOR.message_types_by_name['LogCollectorConfig'] = _LOGCOLLECTORCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogCollectorConfig = _reflection.GeneratedProtocolMessageType('LogCollectorConfig', (_message.Message,), dict(
  DESCRIPTOR = _LOGCOLLECTORCONFIG,
  __module__ = 'tensorflow_serving.config.log_collector_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.LogCollectorConfig)
  ))
_sym_db.RegisterMessage(LogCollectorConfig)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from tensorflow_serving.apis import classification_pb2 as tensorflow__serving_dot_apis_dot_classification__pb2
from tensorflow_serving.apis import get_model_metadata_pb2 as tensorflow__serving_dot_apis_dot_get__model__metadata__pb2
from tensorflow_serving.apis import inference_pb2 as tensorflow__serving_dot_apis_dot_inference__pb2
from tensorflow_serving.apis import predict_pb2 as tensorflow__serving_dot_apis_dot_predict__pb2
from tensorflow_serving.apis import regression_pb2 as tensorflow__serving_dot_apis_dot_regression__pb2


class PredictionServiceStub(object):
  """open source marker; do not remove
  PredictionService provides access to machine-learned models loaded by
  model_servers.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Classify = channel.unary_unary(
        '/tensorflow.serving.PredictionService/Classify',
        request_serializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.SerializeToString,
        response_deserializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.FromString,
        )
    self.Regress = channel.unary_unary(
        '/tensorflow.serving.PredictionService/Regress',
        request_serializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.SerializeToString,
        response_deserializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.FromString,
        )
    self.Predict = channel.unary_unary(
        '/tensorflow.serving.PredictionService/Predict',
        request_serializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.SerializeToString,
        response_deserializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.FromString,
        )
    self.MultiInference = channel.unary_unary(
        '/tensorflow.serving.PredictionService/MultiInference',
        request_serializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.SerializeToString,
        response_deserializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.FromString,
        )
    self.GetModelMetadata = channel.unary_unary(
        '/tensorflow.serving.PredictionService/GetModelMetadata',
        request_serializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.SerializeToString,
        response_deserializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.FromString,
        )


class PredictionServiceServicer(object):
  """open source marker; do not remove
  PredictionService provides access to machine-learned models loaded by
  model_servers.
  """

  def Classify(self, request, context):
    """Classify.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Regress(self, request, context):
    """Regress.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Predict(self, request, context):
    """Predict -- provides access to loaded TensorFlow model.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def MultiInference(self, request, context):
    """MultiInference API for multi-headed models.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetModelMetadata(self, request, context):
    """GetModelMetadata - provides access to metadata for loaded models.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_PredictionServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Classify': grpc.unary_unary_rpc_method_handler(
          servicer.Classify,
          request_deserializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationRequest.FromString,
          response_serializer=tensorflow__serving_dot_apis_dot_classification__pb2.ClassificationResponse.SerializeToString,
      ),
      'Regress': grpc.unary_unary_rpc_method_handler(
          servicer.Regress,
          request_deserializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionRequest.FromString,
          response_serializer=tensorflow__serving_dot_apis_dot_regression__pb2.RegressionResponse.SerializeToString,
      ),
      'Predict': grpc.unary_unary_rpc_method_handler(
          servicer.Predict,
          request_deserializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictRequest.FromString,
          response_serializer=tensorflow__serving_dot_apis_dot_predict__pb2.PredictResponse.SerializeToString,
      ),
      'MultiInference': grpc.unary_unary_rpc_method_handler(
          servicer.MultiInference,
          request_deserializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceRequest.FromString,
          response_serializer=tensorflow__serving_dot_apis_dot_inference__pb2.MultiInferenceResponse.SerializeToString,
      ),
      'GetModelMetadata': grpc.unary_unary_rpc_method_handler(
          servicer.GetModelMetadata,
          request_deserializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataRequest.FromString,
          response_serializer=tensorflow__serving_dot_apis_dot_get__model__metadata__pb2.GetModelMetadataResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'tensorflow.serving.PredictionService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
from modelhub.core.models import Model
from .base import BaseCommand, argument, option, register


@register("delete")
class Command(BaseCommand):

    arguments = [
        argument("model_name"),
        option("-r", "--remote", is_flag=True, help='CAUTION! This removes your model from modelhub remotely.'),
        option("-f", "--force", is_flag=True, help='CAUTION! This removes your model permantly.')
    ]

    def run(self, model_name, remote=False, force=False):
        """Delete your model locally or remotely."""
        model_name, version = Model.split_name_version(model_name)
        username, email = self.get_user_info()

        if remote:
            model = Model.get(name=model_name)
            if not version:
                if email == model.manifest.owner_email:
                    if force or input("{count} version of model {model.name} will be removed permanently, confirm delete? Input 'yes':\n>>".format(model=model, count=len(model.versions))) == "yes":
                        model.delete()
                else:
                    self.echo("You have no permission to delete model {model.name}".format(model=model))
            else:
                model_version = model.get_version(version)
                if email != model.owner_email and email != version:
                    self.echo("You have no permission to delete model {model.name}@{version}".format(model=model, version=version))
                model_version.delete()
        else:
            model = Model(name=model_name)
            if not version:
                if force or input("all local versions of model {model.name} will be removed, confirm delete? Input 'yes':\n>>".format(
                    model=model,
                    # count=len([model.versions for )
                )) == "yes":
                    model.delete_local()
            else:
                model_version = model.get_version(version)
                model_version.delete_local()



# encoding=utf8
# author=spenly
# mail=i@spenly.com

from abc import ABCMeta, abstractmethod, abstractproperty
import logging
import logging.config
from modelhub.utils import json
from modelhub.core import conf


class ApiModel(metaclass=ABCMeta):
    TYPE = "NTF"  # choose from 'TF' or 'NTF': TensorFlow-based or not

    INPUTS_SAMPLE = None
    OUTPUTS_SAMPLE = None

    @abstractproperty
    def model_name(self):
        raise NotImplementedError

    def __init__(self, verbose=False, **kwargs):
        self.__dict__.update(kwargs)
        self.verbose = verbose
        logging.config.dictConfig(conf.LOG_CONFIG)
        self.logger = logging.getLogger("api_model")
        if self.verbose:
            self.logger.setLevel(logging.DEBUG)
        self.prepare()

    @abstractmethod
    def prepare(self):
        """
        # must have, rewrite
        prepare models/datasets only once
        :return:
        """
        pass

    @abstractmethod
    def is_ready(self):
        """
        # must have, rewrite
        check preparation above before running
        :return: True or False
        """
        pass

    def preprocess(self, raw_input):
        """
        # optional
        preprocess data
        :param raw_input: input data in a dict format (may have a nested structure in values) from API Platform
        :return: preprocessed data, define data structure as you prefer
        """
        return raw_input

    @abstractmethod
    def run_model(self, preprocessed_data):
        """
        # must have, rewrite
        run model to do inference
        :param preprocessed_data: preprocessed data
        :return: inferred data, define data structure in your model. We recommend using a dict structure
            (may have a one-layer nested structure in values) to store results.
            This may output to API Platform directly without post-processing.
        """
        raise NotImplementedError

    def postprocess(self, result, raw_input, preprocessed_data):
        """
        # optional
        postprocess inferred data
        :param result: result
        :param raw_input: user input before preprocessing
        :param preprocessed_data: input after preprocessing
        :return: output data in a dict format (may have a one-layer nested structure in values) to API Platform
        """
        return result

    def run(self, raw_input):
        """
        # must have
        run function
        :param raw_input: data
        :return: result
        The format of raw_input and result please refer to docstring()
        """
        self.validate_input_data(raw_input)
        preprocessed_data = self.preprocess(raw_input)
        inferenced_data = self.run_model(preprocessed_data)
        return self.postprocess(inferenced_data, raw_input, preprocessed_data)

    def _get_data_json_type(self, data):
        dict_type = type(data).__name__.lower()
        type_map = {
            "dict": "object",
            "list": "array",
            "decimal": "number",
            "int": "nunber",
            "float": "number"
        }
        return dict_type in type_map and type_map[dict_type] or dict_type

    def docstring(self):
        '''
        # must have, rewrite
        docstring for running function
        :return: docs
        Example:
        docs = """
        inputs:
            type: string
            description: text string
            default:
                        text: "your input string"
        outputs:
            type: array
            description: result list
            default:
                -   val1: "value"
                    val2: 2
        """

        ## or return a dict:
        docs = {
            "inputs": {"type": "", "description": "inputs sample", "default": ""},
            "outputs": {"type": "", "description": "outputs sample", "default": ""}
        }
        '''

        docs = """
        inputs:
        outputs:
        """
        if self.INPUTS_SAMPLE and self.OUTPUTS_SAMPLE:
            docs = {
                "inputs": {"type": self._get_data_json_type(self.INPUTS_SAMPLE),
                           "description": "inputs sample",
                           "default": self.INPUTS_SAMPLE
                           },
                "outputs": {"type": self._get_data_json_type(self.OUTPUTS_SAMPLE),
                            "description": "outputs sample",
                            "default": self.OUTPUTS_SAMPLE}
            }
        return docs

    class InvalidValueInput(Exception):
        pass

    @abstractmethod
    def validate_input_data(self, raw_input):
        "raise BaseModel.InvalidValueInput if input is not expected"
        pass

    def log_info(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.info()
        """
        self.logger.info(msg, *args, **kwargs)

    def log_error(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.error()
        """
        self.logger.error(msg, *args, **kwargs)

    def log_debug(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.debug()
        """
        self.logger.debug(msg, *args, **kwargs)

    def log_warning(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.warning()
        """
        self.logger.warning(msg, *args, **kwargs)

    def log_exception(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.exception()
        """
        self.logger.exception(msg, *args, **kwargs)

    def log_critical(self, msg, *args, **kwargs):
        """
        log a message, more details to see logging.critical()
        """
        self.logger.critical(msg, *args, **kwargs)

    def _run_dict(self, data):
        result = self.run(data)
        return json.loads(json.dumps(result))


class BaseModel(ApiModel):
    def __init__(self, *args, **kwargs):
        self.log_warning("framework.BaseModel/TFBaseModel has deprecated, please use framework.APIModel/TFAPIModel instead")
        super().__init__(*args, **kwargs)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_serving/config/platform_config.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_serving/config/platform_config.proto',
  package='tensorflow.serving',
  syntax='proto3',
  serialized_pb=_b('\n/tensorflow_serving/config/platform_config.proto\x12\x12tensorflow.serving\x1a\x19google/protobuf/any.proto\"E\n\x0ePlatformConfig\x12\x33\n\x15source_adapter_config\x18\x01 \x01(\x0b\x32\x14.google.protobuf.Any\"\xc5\x01\n\x11PlatformConfigMap\x12T\n\x10platform_configs\x18\x01 \x03(\x0b\x32:.tensorflow.serving.PlatformConfigMap.PlatformConfigsEntry\x1aZ\n\x14PlatformConfigsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\".tensorflow.serving.PlatformConfig:\x02\x38\x01\x42\x03\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_PLATFORMCONFIG = _descriptor.Descriptor(
  name='PlatformConfig',
  full_name='tensorflow.serving.PlatformConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source_adapter_config', full_name='tensorflow.serving.PlatformConfig.source_adapter_config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=98,
  serialized_end=167,
)


_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY = _descriptor.Descriptor(
  name='PlatformConfigsEntry',
  full_name='tensorflow.serving.PlatformConfigMap.PlatformConfigsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tensorflow.serving.PlatformConfigMap.PlatformConfigsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorflow.serving.PlatformConfigMap.PlatformConfigsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=277,
  serialized_end=367,
)

_PLATFORMCONFIGMAP = _descriptor.Descriptor(
  name='PlatformConfigMap',
  full_name='tensorflow.serving.PlatformConfigMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='platform_configs', full_name='tensorflow.serving.PlatformConfigMap.platform_configs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=367,
)

_PLATFORMCONFIG.fields_by_name['source_adapter_config'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY.fields_by_name['value'].message_type = _PLATFORMCONFIG
_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY.containing_type = _PLATFORMCONFIGMAP
_PLATFORMCONFIGMAP.fields_by_name['platform_configs'].message_type = _PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY
DESCRIPTOR.message_types_by_name['PlatformConfig'] = _PLATFORMCONFIG
DESCRIPTOR.message_types_by_name['PlatformConfigMap'] = _PLATFORMCONFIGMAP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PlatformConfig = _reflection.GeneratedProtocolMessageType('PlatformConfig', (_message.Message,), dict(
  DESCRIPTOR = _PLATFORMCONFIG,
  __module__ = 'tensorflow_serving.config.platform_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.PlatformConfig)
  ))
_sym_db.RegisterMessage(PlatformConfig)

PlatformConfigMap = _reflection.GeneratedProtocolMessageType('PlatformConfigMap', (_message.Message,), dict(

  PlatformConfigsEntry = _reflection.GeneratedProtocolMessageType('PlatformConfigsEntry', (_message.Message,), dict(
    DESCRIPTOR = _PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY,
    __module__ = 'tensorflow_serving.config.platform_config_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.serving.PlatformConfigMap.PlatformConfigsEntry)
    ))
  ,
  DESCRIPTOR = _PLATFORMCONFIGMAP,
  __module__ = 'tensorflow_serving.config.platform_config_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.serving.PlatformConfigMap)
  ))
_sym_db.RegisterMessage(PlatformConfigMap)
_sym_db.RegisterMessage(PlatformConfigMap.PlatformConfigsEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY.has_options = True
_PLATFORMCONFIGMAP_PLATFORMCONFIGSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
from .base import BaseCommand, argument, option, types, register  # noqa
from modelhub.core.models import Model, ModelVersion
try:
    from tensorflow.core.framework import types_pb2
except:
    from tensorflow_protos.core.framework import types_pb2


@register("info")
class Command(BaseCommand):

    arguments = [
        argument("model_path_or_name"),
    ]

    def run(self, model_path_or_name):
        """
        Get info of a model
        """

        if self._is_model_path(model_path_or_name):
            self.local_model_info(model_path_or_name)
        else:
            model_name, version = Model.split_name_version(model_path_or_name)
            model = Model.get(name=model_name)
            if version is None:
                self.remote_list_version(model)
            else:
                self.remote_version_info(model.get_version(version))

    def _is_model_path(self, model_path_or_name):

        if "/" in model_path_or_name:
            return True
        if self._validate_path(model_path_or_name):
            return True

        return False

    def _validate_path(self, local_path):
        return os.path.exists(local_path)

    def local_model_info(self, model_path):
        # from tensorflow.contrib.saved_model.python.saved_model import reader
        # from tensorflow.python.tools.saved_model_cli import _show_all
        # saved_model = reader.read_saved_model(model_path)
        try:
            self._show_all(ModelVersion._load_saved_model(model_path))
        except OSError as e:
            print(e)
            print("Hint: perhaps forget version? Try add version_number")

    def remote_list_version(self, model):
        self._print_model_info(model)
        print("Versions:")
        for version in (model.versions):
            print("""id:\t{version.seq}
is_saved_model:\t{version.is_saved_model}
local:\t{is_local}
user:\t{version.submit_username}<{version.submit_email}>
date:\t{version_datetime}
comment:\t{version.comment}
require GPU:\t{version.require_gpu}
size:\t{version.compressed_size:,}
            """.format(
                model=model.manifest,
                version=version.manifest,
                version_datetime=version.manifest.submit_datetime.ToDatetime(),
                is_local=version.local_exists and version.local_path),
            )

    def _print_model_info(self, model):
        print("""name:\t{model.name}
desc:\t{model.description}
owner:\t{model.owner_name}<{model.owner_email}>
            """.format(model=model.manifest))

    def remote_version_info(self, version):
        self._print_model_info(version.model)
        print(version.manifest)

    def _show_all(self, saved_model):
        """Prints tag-set, SignatureDef and Inputs/Outputs information in SavedModel.

        Prints all tag-set, SignatureDef and Inputs/Outputs information stored in
        SavedModel directory.

        Args:
          saved_model: Directory containing the SavedModel to inspect.
        """
        tag_sets = self._get_saved_model_tag_sets(saved_model)
        for tag_set in sorted(tag_sets):
            tag_set = ', '.join(tag_set)
            print('\nMetaGraphDef with tag-set: \'' + tag_set +
                  '\' contains the following SignatureDefs:')

            signature_def_map = self._get_signature_def_map(saved_model, tag_set)
            for signature_def_key in sorted(signature_def_map.keys()):
                print('\nsignature_def[\'' + signature_def_key + '\']:')
                self._show_inputs_outputs(saved_model, tag_set, signature_def_key)

    def _get_saved_model_tag_sets(self, saved_model):
        """Retrieves all the tag-sets available in the SavedModel.

        Args:
          saved_model: Directory containing the SavedModel.

        Returns:
          String representation of all tag-sets in the SavedModel.
        """
        all_tags = []
        for meta_graph_def in saved_model.meta_graphs:
            all_tags.append(list(meta_graph_def.meta_info_def.tags))
        return all_tags

    def _get_signature_def_map(self, saved_model, tag_set):
        """Gets SignatureDef map from a MetaGraphDef in a SavedModel.

        Returns the SignatureDef map for the given tag-set in the SavedModel
        directory.

        Args:
          saved_model: Directory containing the SavedModel to inspect or execute.
          tag_set: Group of tag(s) of the MetaGraphDef with the SignatureDef map, in
              string format, separated by ','. For tag-set contains multiple tags, all
              tags must be passed in.

        Returns:
          A SignatureDef map that maps from string keys to SignatureDefs.
        """
        meta_graph = self._get_meta_graph_def(saved_model, tag_set)
        return meta_graph.signature_def

    def _get_meta_graph_def(self, saved_model, tag_set):
        """Gets MetaGraphDef from SavedModel.

        Returns the MetaGraphDef for the given tag-set and SavedModel directory.

        Args:
          saved_model: Directory containing the SavedModel to inspect or execute.
          tag_set: Group of tag(s) of the MetaGraphDef to load, in string format,
              separated by ','. For tag-set contains multiple tags, all tags must be
              passed in.

        Raises:
          RuntimeError: An error when the given tag-set does not exist in the
              SavedModel.

        Returns:
          A MetaGraphDef corresponding to the tag-set.
        """
        set_of_tags = set(tag_set.split(','))
        for meta_graph_def in saved_model.meta_graphs:
            if set(meta_graph_def.meta_info_def.tags) == set_of_tags:
                return meta_graph_def

        raise RuntimeError('MetaGraphDef associated with tag-set ' + tag_set +
                           ' could not be found in SavedModel')

    def _show_inputs_outputs(self, saved_model, tag_set, signature_def_key):
        """Prints input and output TensorInfos.

        Prints the details of input and output TensorInfos for the SignatureDef mapped
        by the given signature_def_key.

        Args:
          saved_model: Directory containing the SavedModel to inspect.
          tag_set: Group of tag(s) of the MetaGraphDef, in string format, separated by
              ','. For tag-set contains multiple tags, all tags must be passed in.
          signature_def_key: A SignatureDef key string.
        """
        meta_graph_def = self._get_meta_graph_def(saved_model, tag_set)
        inputs_tensor_info = self._get_inputs_tensor_info_from_meta_graph_def(
            meta_graph_def, signature_def_key)
        outputs_tensor_info = self._get_outputs_tensor_info_from_meta_graph_def(
            meta_graph_def, signature_def_key)

        print('The given SavedModel SignatureDef contains the following input(s):')
        for input_key, input_tensor in sorted(inputs_tensor_info.items()):
            print('inputs[\'%s\'] tensor_info:' % input_key)
            self._print_tensor_info(input_tensor)

        print('The given SavedModel SignatureDef contains the following output(s):')
        for output_key, output_tensor in sorted(outputs_tensor_info.items()):
            print('outputs[\'%s\'] tensor_info:' % output_key)
            self._print_tensor_info(output_tensor)

        print('Method name is: %s' %
              meta_graph_def.signature_def[signature_def_key].method_name)

    def _get_inputs_tensor_info_from_meta_graph_def(self, meta_graph_def,
                                                    signature_def_key):
        """Gets TensorInfo for all inputs of the SignatureDef.

        Returns a dictionary that maps each input key to its TensorInfo for the given
        signature_def_key in the meta_graph_def

        Args:
          meta_graph_def: MetaGraphDef protocol buffer with the SignatureDef map to
              look up SignatureDef key.
          signature_def_key: A SignatureDef key string.

        Returns:
          A dictionary that maps input tensor keys to TensorInfos.
        """
        return self._get_signature_def_by_key(meta_graph_def, signature_def_key).inputs

    def _get_outputs_tensor_info_from_meta_graph_def(self, meta_graph_def,
                                                     signature_def_key):
        """Gets TensorInfos for all outputs of the SignatureDef.

        Returns a dictionary that maps each output key to its TensorInfo for the given
        signature_def_key in the meta_graph_def.

        Args:
          meta_graph_def: MetaGraphDef protocol buffer with the SignatureDefmap to
          look up signature_def_key.
          signature_def_key: A SignatureDef key string.

        Returns:
          A dictionary that maps output tensor keys to TensorInfos.
        """
        return self._get_signature_def_by_key(meta_graph_def, signature_def_key).outputs

    def _print_tensor_info(self, tensor_info):
        """Prints details of the given tensor_info.

        Args:
          tensor_info: TensorInfo object to be printed.
        """
        print('    dtype: ' + types_pb2.DataType.keys()[tensor_info.dtype])
        # Display shape as tuple.
        if tensor_info.tensor_shape.unknown_rank:
            shape = 'unknown_rank'
        else:
            dims = [str(dim.size) for dim in tensor_info.tensor_shape.dim]
            shape = ', '.join(dims)
            shape = '(' + shape + ')'
        print('    shape: ' + shape)
        print('    name: ' + tensor_info.name)

    def _get_signature_def_by_key(self, meta_graph_def, signature_def_key):
      """Utility function to get a SignatureDef protocol buffer by its key.

      Args:
        meta_graph_def: MetaGraphDef protocol buffer with the SignatureDefMap to
          look up.
        signature_def_key: Key of the SignatureDef protocol buffer to find in the
          SignatureDefMap.

      Returns:
        A SignatureDef protocol buffer corresponding to the supplied key, if it
        exists.

      Raises:
        ValueError: If no entry corresponding to the supplied key is found in the
        SignatureDefMap of the MetaGraphDef.
      """
      if signature_def_key not in meta_graph_def.signature_def:
        raise ValueError("No SignatureDef with key '%s' found in MetaGraphDef." %
                         signature_def_key)
      return meta_graph_def.signature_def[signature_def_key]



# -*- coding: utf-8 -*-
import json
import datetime
import uuid
import decimal
import numpy as np


class JSONEncoder(json.JSONEncoder):

    DATE_FORMAT = "%Y-%m-%d"
    TIME_FORMAT = "%H:%M:%S"

    @staticmethod
    def safe_new_datetime(d):
        kw = [d.year, d.month, d.day]
        if isinstance(d, datetime.datetime):
            kw.extend([d.hour, d.minute, d.second, d.microsecond, d.tzinfo])
        return datetime.datetime(*kw)

    @staticmethod
    def safe_new_date(d):
        return datetime.date(d.year, d.month, d.day)

    def default(self, o):
        if isinstance(o, np.ndarray):
            return o.tolist()
        if isinstance(o, np.floating):
            return float(o)
        if isinstance(o, np.integer):
            return int(o)
        if isinstance(o, bytes):
            return o.decode()
        if isinstance(o, set):
            return list(o)
        if isinstance(o, datetime.datetime):
            d = self.safe_new_datetime(o)
            return d.isoformat(" ")
        if isinstance(o, datetime.date):
            d = self.safe_new_date(o)
            return d.strftime(self.DATE_FORMAT)
        if isinstance(o, datetime.time):
            return o.strftime(self.TIME_FORMAT)
        if isinstance(o, datetime.timedelta):
            return o.total_seconds()
        if isinstance(o, decimal.Decimal):
            return str(o)
        if isinstance(o, uuid.UUID):
            return str(o)
        return super(JSONEncoder, self).default(o)


def dumps(*args, **kwargs):
    kwargs.setdefault("cls", JSONEncoder)
    return json.dumps(*args, **kwargs)


def dump(*args, **kwargs):
    kwargs.setdefault("cls", JSONEncoder)
    return json.dump(*args, **kwargs)


load = json.load

loads = json.loads



# -*- coding: utf-8 -*-
from modelhub.core.models import Model
from .base import BaseCommand, argument, option, register


@register("create")
class Command(BaseCommand):

    arguments = [
        argument("name"),
        option("-m", "--desc", prompt=True, help='describe your model here.')
    ]

    def run(self, name, desc):
        """Create a new model."""
        try:
            Model.get(name=name)
        except Model.DoesNotExist:
            pass
        else:
            self.echo("Model {name} already exists".format(name=name))
            return
        Model.create(name, desc, *self.get_user_info())
